<?php

namespace App\Http\Controllers;

use App\Models\Brand;
use App\Models\Car;
use App\Models\Category;
use Illuminate\Support\Facades\Session;

class BrandsController extends Controller
{

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    
    public function brand_details($slug) {
        // Get the previous and current URLs
        $previous = url()->previous();
        $current = url()->current();

        // Check if the base URLs are different
        $isFromAnotherPage =  $previous != $current;

        if ($isFromAnotherPage) {
            // The current request is from another page
            // Delete the session
            Session::forget('filter_sortByCars');
            Session::forget('filter_availability');
        } else {
            // The current request is from the same page
        }

        $type = session('filter_sortByPrice');
        $selectedCars = session('filter_sortByCars', []);
        $availability = session('filter_availability');

        $brand = Brand::where('slug', $slug)->first();
        $brands = Brand::select('id', 'name_en', 'name_ar', 'slug', 'logo')->get();
        $brand_id = $brand->id;

        $cars = Car::where('brand_id', $brand_id)->when($type == 'hightolow', function ($query) {
            $query->orderByRaw('CAST(price_daily AS UNSIGNED) DESC');
        })
        ->when($type == 'lowtohigh', function ($query) {
            $query->orderByRaw('CAST(price_daily AS UNSIGNED) ASC');
        })
        ->when(count($selectedCars) >= 1, function ($query) use ($selectedCars) {
            $query->whereIn('category_id', $selectedCars);
        })
        ->when($availability != null, function ($query) use ($availability) {
            $query->where('stock', $availability == 'in-stock' ? 1 : 0);
        })->orderBy('model', 'desc')->paginate(6);

        $categories = Category::select('id', 'name_en', 'name_ar')
            ->withCount(['cars' => function ($query) use ($brand_id) {
                $query->where('brand_id', $brand_id);
            }])->get();

        return view('brands.details')->with(['brand' => $brand,'brands' => $brands, 'cars' => $cars, 'categories' => $categories]);
        
    }

}
