<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use App\Models\Car;
use App\Models\Brand;
use App\Models\Category;


// use Illuminate\Http\Request;

class CarsController extends Controller
{

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    
    public function index() {
        // Get the previous and current URLs
        $previous = url()->previous();
        $current = url()->current();

        // Check if the base URLs are different
        $isFromAnotherPage =  $previous != $current;

        if ($isFromAnotherPage) {
            // The current request is from another page
            // Delete the session
            Session::forget('filter_sortByCars');
            Session::forget('filter_availability');
        } else {
            // The current request is from the same page
        }

        $type = session('filter_sortByPrice');
        $selectedCars = session('filter_sortByCars', []);
        $availability = session('filter_availability');

        $brands = Brand::select('id', 'name_en', 'name_ar', 'slug', 'logo')->get();
        
        $cars = Car::when($type == 'hightolow', function ($query) {
                $query->orderByRaw('CAST(price_daily AS UNSIGNED) DESC');
            })
            ->when($type == 'lowtohigh', function ($query) {
                $query->orderByRaw('CAST(price_daily AS UNSIGNED) ASC');
            })
            ->when(count($selectedCars) >= 1, function ($query) use ($selectedCars) {
                $query->whereIn('category_id', $selectedCars);
            })
            ->when($availability != null, function ($query) use ($availability) {
                $query->where('stock', $availability == 'in-stock' ? 1 : 0);
            })->orderBy('model', 'desc')
            ->paginate(12); // 12 cars per page
        

        // Get categories with counts product
        $categories = Category::select('id', 'name_en', 'name_ar')
            ->withCount(['cars' => function ($query) use ($brands) {
            }])->get();

        return view('cars.index')->with(['brands' => $brands, 'cars' => $cars, 'categories' => $categories]);
        
    }

    public function car_details($slug){

        $car = Car::where('slug',$slug)->first();
        $brands = Brand::select('id','name_en', 'name_ar', 'slug', 'logo')->get();

        $car_images = json_decode($car->images, true);

        if (!$car) {
            abort(404);
        }

        return view('cars.details')->with(['brands' => $brands, 'car' => $car, 'car_images' => $car_images]);
        
    }

    // Filter
    public function Filter(Request $request)
    {
        if (isset($request['type'])) {
            $type = $request['type'];
            session(['filter_sortByPrice' => $type]);
        }

        if (isset($request['checkboxes'])) {
            $checkboxes = $request['checkboxes'];
            session(['filter_sortByCars' => $checkboxes]);
        }

        if (isset($request['isNull'])) {
            if ($request['isNull'] == "true") {
                // Remove the 'filter_sortByCars' key from the session
                Session::forget('filter_sortByCars');
            }
        }

        if (isset($request['availability'])) {
            $availability = $request['availability'];
            session(['filter_availability' => $availability]);
        }

        $tt = session('filter_availability');

        return response()->json([
            'message' => "success => $tt"
        ]);
    }

}
